@@ -23,6 +23,8 @@ L{Transport} is a subclass of L{BaseTransport} that handles authentication.
 This separation keeps either class file from being too unwieldy.
 """
 
+import threading
+
 # this helps freezing utils
 import encodings.utf_8
 
@@ -30,7 +32,7 @@ from common import *
 import util
 from transport import BaseTransport
 from message import Message
-from ssh_exception import SSHException
+from ssh_exception import SSHException, BadAuthenticationType, PartialAuthentication
 
 
 class Transport (BaseTransport):
@@ -47,8 +49,6 @@ class Transport (BaseTransport):
     another shell window).
     """
     
-    AUTH_SUCCESSFUL, AUTH_PARTIALLY_SUCCESSFUL, AUTH_FAILED = range(3)
-
     def __init__(self, sock):
         BaseTransport.__init__(self, sock)
         self.username = None
@@ -57,23 +57,24 @@ class Transport (BaseTransport):
         # for server mode:
         self.auth_username = None
         self.auth_fail_count = 0
-        self.auth_complete = 0
 
     def __repr__(self):
+        out = '<paramiko.Transport at %s' % hex(id(self))
         if not self.active:
-            return '<paramiko.Transport (unconnected)>'
-        out = '<paramiko.Transport'
-        if self.local_cipher != '':
-            out += ' (cipher %s, %d bits)' % (self.local_cipher, self._cipher_info[self.local_cipher]['key-size'] * 8)
-        if self.authenticated:
-            if len(self.channels) == 1:
-                out += ' (active; 1 open channel)'
-            else:
-                out += ' (active; %d open channels)' % len(self.channels)
-        elif self.initial_kex_done:
-            out += ' (connected; awaiting auth)'
+            out += ' (unconnected)'
         else:
-            out += ' (connecting)'
+            if self.local_cipher != '':
+                out += ' (cipher %s, %d bits)' % (self.local_cipher,
+                                                  self._cipher_info[self.local_cipher]['key-size'] * 8)
+            if self.authenticated:
+                if len(self.channels) == 1:
+                    out += ' (active; 1 open channel)'
+                else:
+                    out += ' (active; %d open channels)' % len(self.channels)
+            elif self.initial_kex_done:
+                out += ' (connected; awaiting auth)'
+            else:
+                out += ' (connecting)'
         out += '>'
         return out
 
@@ -98,14 +99,28 @@ class Transport (BaseTransport):
 
         @since: fearow
         """
-        return self.username
+        if self.server_mode:
+            return self.auth_username
+        else:
+            return self.username
 
-    def auth_publickey(self, username, key, event):
+    def auth_publickey(self, username, key, event=None):
         """
         Authenticate to the server using a private key.  The key is used to
-        sign data from the server, so it must include the private part.  The
-        given L{event} is triggered on success or failure.  On success,
-        L{is_authenticated} will return C{True}.
+        sign data from the server, so it must include the private part.
+        
+        If an C{event} is passed in, this method will return immediately, and
+        the event will be triggered once authentication succeeds or fails.  On
+        success, L{is_authenticated} will return C{True}.  On failure, you may
+        use L{get_exception} to get more detailed error information.
+        
+        Since 1.1, if no event is passed, this method will block until the
+        authentication succeeds or fails.  On failure, an exception is raised.
+        Otherwise, the method simply returns.
+
+        If the server requires multi-step authentication (which is very rare),
+        this method will return a list of auth types permissible for the next
+        step.  Otherwise, in the normal case, an empty list is returned.
 
         @param username: the username to authenticate as.
         @type username: string
@@ -114,27 +129,54 @@ class Transport (BaseTransport):
         @param event: an event to trigger when the authentication attempt is
         complete (whether it was successful or not)
         @type event: threading.Event
+        @return: list of auth types permissible for the next stage of
+            authentication (normally empty).
+        @rtype: list
+        
+        @raise BadAuthenticationType: if public-key authentication isn't
+            allowed by the server for this user (and no event was passed in).
+        @raise SSHException: if the authentication failed (and no event was
+            passed in).
         """
         if (not self.active) or (not self.initial_kex_done):
             # we should never try to authenticate unless we're on a secure link
             raise SSHException('No existing session')
+        if event is None:
+            my_event = threading.Event()
+        else:
+            my_event = event
+        self.lock.acquire()
         try:
-            self.lock.acquire()
-            self.auth_event = event
+            self.auth_event = my_event
             self.auth_method = 'publickey'
             self.username = username
             self.private_key = key
             self._request_auth()
         finally:
             self.lock.release()
+        if event is not None:
+            # caller wants to wait for event themselves
+            return []
+        return self._wait_for_response(my_event)
 
-    def auth_password(self, username, password, event):
+    def auth_password(self, username, password, event=None):
         """
         Authenticate to the server using a password.  The username and password
-        are sent over an encrypted link, and the given L{event} is triggered on
-        success or failure.  On success, L{is_authenticated} will return
-        C{True}.
+        are sent over an encrypted link.
+        
+        If an C{event} is passed in, this method will return immediately, and
+        the event will be triggered once authentication succeeds or fails.  On
+        success, L{is_authenticated} will return C{True}.  On failure, you may
+        use L{get_exception} to get more detailed error information.
 
+        Since 1.1, if no event is passed, this method will block until the
+        authentication succeeds or fails.  On failure, an exception is raised.
+        Otherwise, the method simply returns.
+        
+        If the server requires multi-step authentication (which is very rare),
+        this method will return a list of auth types permissible for the next
+        step.  Otherwise, in the normal case, an empty list is returned.
+        
         @param username: the username to authenticate as.
         @type username: string
         @param password: the password to authenticate with.
@@ -142,115 +184,35 @@ class Transport (BaseTransport):
         @param event: an event to trigger when the authentication attempt is
         complete (whether it was successful or not)
         @type event: threading.Event
+        @return: list of auth types permissible for the next stage of
+            authentication (normally empty).
+        @rtype: list
+        
+        @raise BadAuthenticationType: if password authentication isn't
+            allowed by the server for this user (and no event was passed in).
+        @raise SSHException: if the authentication failed (and no event was
+            passed in).
         """
         if (not self.active) or (not self.initial_kex_done):
             # we should never try to send the password unless we're on a secure link
             raise SSHException('No existing session')
+        if event is None:
+            my_event = threading.Event()
+        else:
+            my_event = event
+        self.lock.acquire()
         try:
-            self.lock.acquire()
-            self.auth_event = event
+            self.auth_event = my_event
             self.auth_method = 'password'
             self.username = username
             self.password = password
             self._request_auth()
         finally:
             self.lock.release()
-
-    def get_allowed_auths(self, username):
-        """
-        I{(subclass override)}
-        Return a list of authentication methods supported by the server.
-        This list is sent to clients attempting to authenticate, to inform them
-        of authentication methods that might be successful.
-
-        The "list" is actually a string of comma-separated names of types of
-        authentication.  Possible values are C{"password"}, C{"publickey"},
-        and C{"none"}.
-
-        The default implementation always returns C{"password"}.
-
-        @param username: the username requesting authentication.
-        @type username: string
-        @return: a comma-separated list of authentication types
-        @rtype: string
-        """
-        return 'password'
-
-    def check_auth_none(self, username):
-        """
-        I{(subclass override)}
-        Determine if a client may open channels with no (further)
-        authentication.  You should override this method in server mode.
-
-        Return C{AUTH_FAILED} if the client must authenticate, or
-        C{AUTH_SUCCESSFUL} if it's okay for the client to not authenticate.
-
-        The default implementation always returns C{AUTH_FAILED}.
-
-        @param username: the username of the client.
-        @type username: string
-        @return: C{AUTH_FAILED} if the authentication fails; C{AUTH_SUCCESSFUL}
-        if it succeeds.
-        @rtype: int
-        """
-        return self.AUTH_FAILED
-
-    def check_auth_password(self, username, password):
-        """
-        I{(subclass override)}
-        Determine if a given username and password supplied by the client is
-        acceptable for use in authentication.  You should override this method
-        in server mode.
-
-        Return C{AUTH_FAILED} if the password is not accepted,
-        C{AUTH_SUCCESSFUL} if the password is accepted and completes the
-        authentication, or C{AUTH_PARTIALLY_SUCCESSFUL} if your authentication
-        is stateful, and this key is accepted for authentication, but more
-        authentication is required.  (In this latter case, L{get_allowed_auths}
-        will be called to report to the client what options it has for
-        continuing the authentication.)
-
-        The default implementation always returns C{AUTH_FAILED}.
-
-        @param username: the username of the authenticating client.
-        @type username: string
-        @param password: the password given by the client.
-        @type password: string
-        @return: C{AUTH_FAILED} if the authentication fails; C{AUTH_SUCCESSFUL}
-        if it succeeds; C{AUTH_PARTIALLY_SUCCESSFUL} if the password auth is
-        successful, but authentication must continue.
-        @rtype: int
-        """
-        return self.AUTH_FAILED
-
-    def check_auth_publickey(self, username, key):
-        """
-        I{(subclass override)}
-        Determine if a given key supplied by the client is acceptable for use
-        in authentication.  You should override this method in server mode to
-        check the username and key and decide if you would accept a signature
-        made using this key.
-
-        Return C{AUTH_FAILED} if the key is not accepted, C{AUTH_SUCCESSFUL}
-        if the key is accepted and completes the authentication, or
-        C{AUTH_PARTIALLY_SUCCESSFUL} if your authentication is stateful, and
-        this key is accepted for authentication, but more authentication is
-        required.  (In this latter case, L{get_allowed_auths} will be called
-        to report to the client what options it has for continuing the
-        authentication.)
-
-        The default implementation always returns C{AUTH_FAILED}.
-
-        @param username: the username of the authenticating client.
-        @type username: string
-        @param key: the key object provided by the client.
-        @type key: L{PKey <pkey.PKey>}
-        @return: C{AUTH_FAILED} if the client can't authenticate with this key;
-        C{AUTH_SUCCESSFUL} if it can; C{AUTH_PARTIALLY_SUCCESSFUL} if it can
-        authenticate with this key but must continue with authentication.
-        @rtype: int
-        """
-        return self.AUTH_FAILED
+        if event is not None:
+            # caller wants to wait for event themselves
+            return []
+        return self._wait_for_response(my_event)
 
 
     ###  internals...
@@ -292,6 +254,27 @@ class Transport (BaseTransport):
         m.add_string(str(key))
         return str(m)
 
+    def _wait_for_response(self, event):
+        while True:
+            event.wait(0.1)
+            if not self.active:
+                e = self.get_exception()
+                if e is None:
+                    e = SSHException('Authentication failed.')
+                raise e
+            if event.isSet():
+                break
+        if not self.is_authenticated():
+            e = self.get_exception()
+            if e is None:
+                e = SSHException('Authentication failed.')
+            # this is horrible.  python Exception isn't yet descended from
+            # object, so type(e) won't work. :(
+            if issubclass(e.__class__, PartialAuthentication):
+                return e.allowed_types
+            raise e
+        return []
+
     def _parse_service_request(self, m):
         service = m.get_string()
         if self.server_mode and (service == 'ssh-userauth'):
@@ -338,7 +321,7 @@ class Transport (BaseTransport):
             m.add_boolean(0)
             self._send_message(m)
             return
-        if self.auth_complete:
+        if self.authenticated:
             # ignore
             return
         username = m.get_string()
@@ -355,30 +338,36 @@ class Transport (BaseTransport):
         self.auth_username = username
 
         if method == 'none':
-            result = self.check_auth_none(username)
+            result = self.server_object.check_auth_none(username)
         elif method == 'password':
             changereq = m.get_boolean()
-            password = m.get_string().decode('UTF-8')
+            password = m.get_string().decode('UTF-8', 'replace')
             if changereq:
                 # always treated as failure, since we don't support changing passwords, but collect
                 # the list of valid auth types from the callback anyway
                 self._log(DEBUG, 'Auth request to change passwords (rejected)')
-                newpassword = m.get_string().decode('UTF-8')
-                result = self.AUTH_FAILED
+                newpassword = m.get_string().decode('UTF-8', 'replace')
+                result = AUTH_FAILED
             else:
-                result = self.check_auth_password(username, password)
+                result = self.server_object.check_auth_password(username, password)
         elif method == 'publickey':
             sig_attached = m.get_boolean()
             keytype = m.get_string()
             keyblob = m.get_string()
-            key = self._key_from_blob(keytype, keyblob)
-            if (key is None) or (not key.valid):
+            try:
+                key = self._key_info[keytype](Message(keyblob))
+            except SSHException, e:
+                self._log(DEBUG, 'Auth rejected: public key: %s' % str(e))
+                key = None
+            except:
                 self._log(DEBUG, 'Auth rejected: unsupported or mangled public key')
+                key = None
+            if key is None:
                 self._disconnect_no_more_auth()
                 return
             # first check if this key is okay... if not, we can skip the verify
-            result = self.check_auth_publickey(username, key)
-            if result != self.AUTH_FAILED:
+            result = self.server_object.check_auth_publickey(username, key)
+            if result != AUTH_FAILED:
                 # key is okay, verify it
                 if not sig_attached:
                     # client wants to know if this key is acceptable, before it
@@ -393,20 +382,20 @@ class Transport (BaseTransport):
                 blob = self._get_session_blob(key, service, username)
                 if not key.verify_ssh_sig(blob, sig):
                     self._log(DEBUG, 'Auth rejected: invalid signature')
-                    result = self.AUTH_FAILED
+                    result = AUTH_FAILED
         else:
-            result = self.check_auth_none(username)
+            result = self.server_object.check_auth_none(username)
         # okay, send result
         m = Message()
-        if result == self.AUTH_SUCCESSFUL:
+        if result == AUTH_SUCCESSFUL:
             self._log(DEBUG, 'Auth granted.')
             m.add_byte(chr(MSG_USERAUTH_SUCCESS))
-            self.auth_complete = 1
+            self.authenticated = True
         else:
             self._log(DEBUG, 'Auth rejected.')
             m.add_byte(chr(MSG_USERAUTH_FAILURE))
-            m.add_string(self.get_allowed_auths(username))
-            if result == self.AUTH_PARTIALLY_SUCCESSFUL:
+            m.add_string(self.server_object.get_allowed_auths(username))
+            if result == AUTH_PARTIALLY_SUCCESSFUL:
                 m.add_boolean(1)
             else:
                 m.add_boolean(0)
@@ -426,14 +415,15 @@ class Transport (BaseTransport):
         partial = m.get_boolean()
         if partial:
             self._log(INFO, 'Authentication continues...')
-            self._log(DEBUG, 'Methods: ' + str(partial))
-            # FIXME - do something
-            pass
-        self._log(INFO, 'Authentication failed.')
+            self._log(DEBUG, 'Methods: ' + str(authlist))
+            self.saved_exception = PartialAuthentication(authlist)
+        elif self.auth_method not in authlist:
+            self._log(INFO, 'Authentication type not permitted.')
+            self.saved_exception = BadAuthenticationType('Bad authentication type', authlist)
+        else:
+            self._log(INFO, 'Authentication failed.')
         self.authenticated = False
-        # FIXME: i don't think we need to close() necessarily here
         self.username = None
-        self.close()
         if self.auth_event != None:
             self.auth_event.set()
 
